﻿using System;
using OpenBveApi.Runtime;

namespace OpenbveFcmbTrainPlugin
{
    /// <summary>A device controlling the door operation.</summary>
    internal class Doors : Device
    {
        /// <summary>Whether the left doors are closing.</summary>
        private bool LeftDoorsClosing;

        /// <summary>Whether the right doors are closing.</summary>
        private bool RightDoorsClosing;

        /// <summary>Is called when the device state should be updated.</summary>
        /// <param name="train">The current train.</param>
        /// <param name="init">Whether the device should initialize.</param>
        /// <param name="elapsedTime">The time elapsed since the previous call.</param>
        internal override void Update(Train train, bool init, Time elapsedTime)
        {
            if (init)
            {
                // Set the selection state of the doors during initialization
                switch (train.DoorState)
                {
                    case DoorStates.None:
                        RequestedDoorInterlock = DoorInterlockStates.Locked;
                        break;
                    case DoorStates.Both:
                        RequestedDoorInterlock = DoorInterlockStates.Unlocked;
                        break;
                    case DoorStates.Left:
                    case DoorStates.Right:
                        RequestedDoorInterlock = (DoorInterlockStates)train.DoorState;
                        break;
                }
            }

            // Cut power when the doors are open
            RequestedPowerNotch = (train.DoorState != DoorStates.None) ? 0 : -1;

            // Update panel variables for door selection state
            train.Panel[50] = (RequestedDoorInterlock == DoorInterlockStates.Left || RequestedDoorInterlock == DoorInterlockStates.Unlocked) ? 1 : 0;
            train.Panel[51] = (RequestedDoorInterlock == DoorInterlockStates.Right || RequestedDoorInterlock == DoorInterlockStates.Unlocked) ? 1 : 0;
        }

        /// <summary>Is called when the state of a key changes.</summary>
        /// <param name="key">The key.</param>
        /// <param name="pressed">Whether the key is pressed or released.</param>
        /// <param name="train">The current train.</param>
        internal override void KeyChange(VirtualKeys key, bool pressed, Train train)
        {
            if (pressed)
            {
                switch (key)
                {
                    case VirtualKeys.G:
                        // Change the selection state of the left doors
                        if (!OpenbveFcmbTrainPlugin.KeysPressed[(int)key])
                        {
                            // Button active only if the doors are not open
                            if ((train.DoorState & DoorStates.Left) == 0 || LeftDoorsClosing)
                            {
                                switch (RequestedDoorInterlock)
                                {
                                    case DoorInterlockStates.Locked:
                                        RequestedDoorInterlock = DoorInterlockStates.Left;
                                        break;
                                    case DoorInterlockStates.Left:
                                        RequestedDoorInterlock = DoorInterlockStates.Locked;
                                        break;
                                    case DoorInterlockStates.Right:
                                        RequestedDoorInterlock = DoorInterlockStates.Unlocked;
                                        break;
                                    case DoorInterlockStates.Unlocked:
                                        RequestedDoorInterlock = DoorInterlockStates.Right;
                                        break;
                                }
                            }
                        }
                        break;
                    case VirtualKeys.H:
                        // Change the selection state of the right doors
                        if (!OpenbveFcmbTrainPlugin.KeysPressed[(int)key])
                        {
                            // Button active only if the doors are not open
                            if ((train.DoorState & DoorStates.Right) == 0 || RightDoorsClosing)
                            {
                                switch (RequestedDoorInterlock)
                                {
                                    case DoorInterlockStates.Locked:
                                        RequestedDoorInterlock = DoorInterlockStates.Right;
                                        break;
                                    case DoorInterlockStates.Left:
                                        RequestedDoorInterlock = DoorInterlockStates.Unlocked;
                                        break;
                                    case DoorInterlockStates.Right:
                                        RequestedDoorInterlock = DoorInterlockStates.Locked;
                                        break;
                                    case DoorInterlockStates.Unlocked:
                                        RequestedDoorInterlock = DoorInterlockStates.Left;
                                        break;
                                }
                            }
                        }
                        break;
                    case VirtualKeys.LeftDoors:
                        if (!OpenbveFcmbTrainPlugin.KeysPressed[(int)key])
                        {
                            // Unselect doors automatically when closing
                            if ((train.DoorState & DoorStates.Left) > 0 && (RequestedDoorInterlock == DoorInterlockStates.Left || RequestedDoorInterlock == DoorInterlockStates.Unlocked))
                            {
                                LeftDoorsClosing = true;
                                RequestedDoorInterlock = RequestedDoorInterlock == DoorInterlockStates.Left ? DoorInterlockStates.Locked : DoorInterlockStates.Right;
                            }
                        }
                        break;
                    case VirtualKeys.RightDoors:
                        if (!OpenbveFcmbTrainPlugin.KeysPressed[(int)key])
                        {
                            // Unselect doors automatically when closing
                            if ((train.DoorState & DoorStates.Right) > 0 && (RequestedDoorInterlock == DoorInterlockStates.Right || RequestedDoorInterlock == DoorInterlockStates.Unlocked))
                            {
                                RightDoorsClosing = true;
                                RequestedDoorInterlock = RequestedDoorInterlock == DoorInterlockStates.Right ? DoorInterlockStates.Locked : DoorInterlockStates. Left;
                            }
                        }
                        break;
                }
                // TODO: OpenBVE does not trigger virtual buttons when the AI driver is active, needs changes to main program
            }
        }

        /// <summary>Is called when the state of the doors changes.</summary>
        /// <param name="oldState">The old state of the doors.</param>
        /// <param name="newState">The new state of the doors.</param>
        internal override void DoorChange(DoorStates oldState, DoorStates newState)
        {
            LeftDoorsClosing = false;
            RightDoorsClosing = false;
        }

        /// <summary>Is called when the device should perform the AI.</summary>
        /// <param name="data">The AI data.</param>
        /// <param name="train">The current train.</param>
        internal override void PerformAI(AIData data, Train train)
        {
            //// Select the correct doors for the current station
            //if (Stations.State != Stations.StationStates.Completed)
            //{
            //    if (((Stations.Current.OpenLeftDoors && !LeftDoorsSelected) || (!Stations.Current.OpenLeftDoors && LeftDoorsSelected)) && !LeftDoorsClosing)
            //    {
            //        // Select or unselect the doors on the left.
            //        KeyDown(VirtualKeys.G);
            //        data.Response = AIResponse.Short;
            //    }
            //    if (((Stations.Current.OpenRightDoors && !RightDoorsSelected) || (!Stations.Current.OpenRightDoors && RightDoorsSelected)) && !RightDoorsClosing)
            //    {
            //        // Select or unselect the doors on the right.
            //        KeyDown(VirtualKeys.H);
            //        data.Response = AIResponse.Short;
            //    }
            //}
        }
    }
}

